package com.hzs.bonus.bonus.service;

/**
 * @Description: 奖金发放
 * @Author: sui q
 * @Time: 2023/3/20 17:04
 * @Classname: IBonusGrantService
 * @PackageName: com.hzs.member.bonus.service
 */
public interface IBonusGrantService {

    /**
     * 每天公布奖金,定时任务
     * @return: void
     * @Author: sui q
     * @Date: 2023/3/20 17:06
     */
    void publishBonusByPeriodEveryDay();

    void grantWithdrawBonusByPeriodEveryDay();

    /**
     * 手动公布奖金
     * @param settleDate 日期
    * @return: void
    * @Author: sui q
    * @Date: 2023/3/20 17:06
    */
    void publishBonusByPeriod(Integer pkCountry, String settleDate);

    /**
     * 手动不公布奖金
     * @param settleDate 日期
     * @return: void
     * @Author: sui q
     * @Date: 2023/3/20 17:06
     */
    void noPublishBonusByPeriod(Integer pkCountry, String settleDate);

    /**
     * 每天发放奖金
     * @param settleDate 日期
    * @return: void
    * @Author: sui q
    * @Date: 2023/3/20 17:06
    */
    Boolean grantBonusByPeriod(Integer pkCountry, String settleDate);

    void grantBonusByTempPeriod(Integer pkCountry, String settleDate);

    /**
     * 每天奖金提现
     * @param settleDate 日期
    * @return: void
    * @Author: sui q
    * @Date: 2023/3/20 17:07
    */
    void withdrawalBonusByPeriod(Integer pkCountry, String settleDate);
}
